
CREATE TABLE CONTENT (
                    CONTENT_ID VARCHAR (50),
                    CONTENT_DATA BLOB,
                    PRIMARY KEY (CONTENT_ID));

CREATE TABLE RESOURCE (
                    RID VARCHAR (50),
                    PATH VARCHAR (2000) NOT NULL,
                    MEDIA_TYPE VARCHAR (500),
                    COLLECTION INTEGER NOT NULL,
                    CREATOR VARCHAR (500),
                    CREATED_TIME TIMESTAMP,
                    LAST_UPDATOR VARCHAR (500),
                    LAST_UPDATED_TIME TIMESTAMP,
                    DESCRIPTION VARCHAR (500),
                    CONTENT_ID VARCHAR (50),
                    EQUIVALENT_VERSION INTEGER NOT NULL,
                    PRIMARY KEY (RID),
                    FOREIGN KEY (CONTENT_ID) REFERENCES CONTENT (CONTENT_ID),
                    UNIQUE(PATH));

CREATE TABLE DEPENDENCY (
                    DEPENDENCY_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    PARENT_RID VARCHAR (50) NOT NULL,
                    CHILD_RID VARCHAR (50) NOT NULL,
                    PRIMARY KEY (DEPENDENCY_ID),
                    UNIQUE (PARENT_RID, CHILD_RID),
                    FOREIGN KEY (PARENT_RID) REFERENCES RESOURCE (RID) ON DELETE CASCADE,
                    FOREIGN KEY (CHILD_RID) REFERENCES RESOURCE (RID) ON DELETE CASCADE);

CREATE TABLE PROPERTY (
                    PROPERTY_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID VARCHAR (50) NOT NULL,
                    NAME VARCHAR (100) NOT NULL,
                    VALUE VARCHAR (500),
                    PRIMARY KEY (PROPERTY_ID),
                    FOREIGN KEY (RID) REFERENCES RESOURCE (RID) ON DELETE CASCADE);

CREATE TABLE ASSOCIATION (
                    ASSOCIATION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID1 VARCHAR (50) NOT NULL,
                    RID2 VARCHAR (50) NOT NULL,
                    ASSOCIATION_TYPE VARCHAR (20) NOT NULL,
                    UNIQUE (RID1, RID2, ASSOCIATION_TYPE),
                    FOREIGN KEY (RID1) REFERENCES RESOURCE (RID) ON DELETE CASCADE,
                    FOREIGN KEY (RID2) REFERENCES RESOURCE (RID) ON DELETE CASCADE);

CREATE TABLE TAG (
                    TAG_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    TAG_NAME VARCHAR (500) NOT NULL,
                    RID VARCHAR (50) NOT NULL,
                    USER_ID VARCHAR (20) NOT NULL,
                    TAGGED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (TAG_ID),
                    UNIQUE (TAG_NAME, RID, USER_ID));

CREATE TABLE COMMENT (
                    COMMENT_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID VARCHAR (50) NOT NULL,
                    USER_ID VARCHAR (20) NOT NULL,
                    COMMENT_TEXT VARCHAR (500) NOT NULL,
                    COMMENTED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (COMMENT_ID));

CREATE TABLE RATING (
                    RATING_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID VARCHAR (50) NOT NULL,
                    USER_ID VARCHAR (20) NOT NULL,
                    RATING INTEGER NOT NULL,
                    RATED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (RATING_ID));

CREATE TABLE LOG (
                    LOG_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    PATH VARCHAR (2000),
                    USER_ID VARCHAR (20) NOT NULL,
                    LOGGED_TIME TIMESTAMP NOT NULL,
                    ACTION INTEGER NOT NULL,
                    ACTION_DATA VARCHAR (500),
                    PRIMARY KEY (LOG_ID));

CREATE TABLE CONTENT_VERSION (
                    CONTENT_VERSION_ID VARCHAR (50),
                    CONTENT_DATA BLOB,
                    PRIMARY KEY (CONTENT_VERSION_ID));

CREATE TABLE RESOURCE_VERSION (
                    RESOURCE_VERSION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID VARCHAR (50) NOT NULL,
                    VERSION INTEGER NOT NULL,
                    PATH VARCHAR (2000) NOT NULL,
                    MEDIA_TYPE VARCHAR (500),
                    COLLECTION INTEGER NOT NULL,
                    CREATOR VARCHAR (500),
                    CREATED_TIME TIMESTAMP,
                    LAST_UPDATOR VARCHAR (500),
                    LAST_UPDATED_TIME TIMESTAMP,
                    DESCRIPTION VARCHAR (500),
                    CONTENT_ID VARCHAR (50),
                    FOREIGN KEY (CONTENT_ID) REFERENCES CONTENT_VERSION (CONTENT_VERSION_ID),
                    PRIMARY KEY (RESOURCE_VERSION_ID),
                    UNIQUE(RID, VERSION));

CREATE TABLE DEPENDENCY_VERSION (
                    DEPENDENCY_VERSION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    PARENT_RID VARCHAR (50) NOT NULL,
                    PARENT_VERSION INTEGER NOT NULL,
                    CHILD_RID VARCHAR (50) NOT NULL,
                    PRIMARY KEY (DEPENDENCY_VERSION_ID),
                    UNIQUE (PARENT_RID, PARENT_VERSION, CHILD_RID));

CREATE TABLE PROPERTY_VERSION (
                    PROPERTY_VERSION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID VARCHAR (50) NOT NULL,
                    VERSION INTEGER NOT NULL,
                    NAME VARCHAR (100) NOT NULL,
                    VALUE VARCHAR (500),
                    PRIMARY KEY (PROPERTY_VERSION_ID),
                    FOREIGN KEY (RID, VERSION) REFERENCES RESOURCE_VERSION (RID, VERSION));

CREATE TABLE ASSOCIATION_VERSION (
                    ASSOCIATION_VERSION_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    RID1 VARCHAR (50) NOT NULL,
                    RID1_VERSION INTEGER NOT NULL,
                    RID2 VARCHAR (50) NOT NULL,
                    RID2_VERSION INTEGER NOT NULL,
                    ASSOCIATION_TYPE VARCHAR (20) NOT NULL,
                    PRIMARY KEY (ASSOCIATION_VERSION_ID),
                    UNIQUE (RID1, RID1_VERSION, RID2, RID2_VERSION, ASSOCIATION_TYPE),
                    FOREIGN KEY (RID1, RID1_VERSION) REFERENCES RESOURCE_VERSION (RID, VERSION),
                    FOREIGN KEY (RID2, RID2_VERSION) REFERENCES RESOURCE_VERSION (RID, VERSION));

CREATE TABLE SNAPSHOT (
                    SNAPSHOT_ID INTEGER GENERATED BY DEFAULT AS IDENTITY,
                    ROOT_ID VARCHAR (50) NOT NULL,
                    PRIMARY KEY (SNAPSHOT_ID),
                    UNIQUE (SNAPSHOT_ID, ROOT_ID));

CREATE TABLE SNAPSHOT_RESOURCE_VERSION (
                    SRV_ID INTEGER GENERATED ALWAYS AS IDENTITY,
                    SNAPSHOT_ID INTEGER NOT NULL,
                    RID VARCHAR (50) NOT NULL,
                    VERSION INTEGER NOT NULL,
                    UNIQUE (SNAPSHOT_ID, RID, VERSION));